# NOTE: this class has been translated from Java.
# Do not modify this file, changes will be overwritten.

#package orbitgt.mapping3d.sdk.viewer;

# haxe remove begin
from .....com.orbitgis.lib.lang.Boolean import Boolean
from .....com.orbitgis.lib.lang.Double import Double
from .....com.orbitgis.lib.lang.Integer import Integer
# haxe remove end
from .....com.orbitgis.tool.Strings import Strings
from .....orbitgt.mapping3d.sdk.viewer.AList import AList
from .....orbitgt.mapping3d.sdk.viewer.AMap import AMap
from .....orbitgt.mapping3d.sdk.viewer.Bounds import Bounds
from .....orbitgt.mapping3d.sdk.viewer.BridgeFunction import BridgeFunction
from .....orbitgt.mapping3d.sdk.viewer.Collections import Collections
from .....orbitgt.mapping3d.sdk.viewer.Coordinate import Coordinate
from .....orbitgt.mapping3d.sdk.viewer.MapObject import MapObject
from .....orbitgt.mapping3d.sdk.viewer.MeasureState import MeasureState
from .....orbitgt.mapping3d.sdk.viewer.Promise import Promise
from .....orbitgt.mapping3d.sdk.viewer.SerializeUtil import SerializeUtil
from .....orbitgt.mapping3d.sdk.viewer.Viewer3DM import Viewer3DM

#
# Class BridgeViewer implements all Viewer3D methods using a bridge function.
# It is used internally by the SDK to make it easier to port to new platforms.
#
# @version 1.0 November 2018
# @author Erzeel Lander
# @ignore
#
class BridgeViewer(Viewer3DM):
    # The bridge function */
    #private BridgeFunction bridgeFunction

    #
    # Create a new instance.
    # @param applicationName the name of the application that is using the SDK.
    # @param options the map of startup options.
    # @param bridgeFunction the bridge function.
    #
    def __init__(self,applicationName: str,options: AMap[object],bridgeFunction: BridgeFunction):
        super().__init__(applicationName,options)
        self.bridgeFunction = bridgeFunction

    #
    # Update the viewer size when the host window has resized.
    # @param width The new width (pixels).
    # @param height The new height (pixels).
    #
    def setSize(self,width: float,height: float) -> None:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        functionArguments.add(Double(width))
        functionArguments.add(Double(height))
        response = self.bridgeFunction.callFunction("setSize",functionArguments,False)

    #
    # Get the footprint map objects to display on the 2D host reference map. This method should be called after the `onFootprintsChanged` signal has been received.
    # @param viewbounds the spatial bounds of the 2D host reference map.
    # @param scaleDenominator the scale denominator of the 2D host reference map.
    # @return Returns a promise to the footprint objects.
    #
    def getFootprints2D(self,viewbounds: Bounds,scaleDenominator: float) -> Promise[AList[MapObject]]:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        functionArguments.add(viewbounds)
        functionArguments.add(Double(scaleDenominator))
        response = self.bridgeFunction.callFunction("getFootprints2D",functionArguments,True)
        promiseId = response
        promise = Promise[AList[MapObject]]()
        promise.setId(promiseId.intValue())
        return promise

    #
    # Get the field-of-view map objects to display on the host reference map. This method should be called after the `onFieldOfViewsChanged` signal has been received.
    # @return Returns the field-of-view map objects.
    #
    def getFieldOfViews2D(self) -> AList[MapObject]:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        response = self.bridgeFunction.callFunction("getFieldOfViews2D",functionArguments,True)
        return response

    #
    # Get the global map objects to display on the host reference map. This method should be called after the `onGlobalsChanged` signal has been received.
    # @return Returns the global map objects.
    #
    def getGlobals(self) -> AList[MapObject]:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        response = self.bridgeFunction.callFunction("getGlobals",functionArguments,True)
        return response

    #
    # Get the state of the measurement and the map objects to display it on the host reference map. This method should be called after the `onMeasureStateChanged` signal has been received.
    # @return Returns the current measure state or null in case the measure function is not active.
    #
    def getMeasureState(self) -> MeasureState:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        response = self.bridgeFunction.callFunction("getMeasureState",functionArguments,True)
        return response

    #
    # Get the current focus position of the viewer.
    # @return Returns the current focus position (can be null).
    #
    def getFocusPosition(self) -> Coordinate:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        response = self.bridgeFunction.callFunction("getFocusPosition",functionArguments,True)
        return response

    #
    # Set the focus position of the viewer. All views will bring the new focus position in focus.
    # @param position The position to focus on (either 2D or 3D).
    #
    def setFocusPosition(self,position: Coordinate) -> None:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        functionArguments.add(position)
        response = self.bridgeFunction.callFunction("setFocusPosition",functionArguments,False)

    #
    # Set the focus position of the viewer. All views will bring the new focus position in focus.
    # @param position The position to focus on (either 2D or 3D).
    # @param options the available startup options.  `Constants.STARTUP_PAN` and `Constants.STARTUP_TILT` are supported.
    #
    def setFocusPositionWithOptions(self,position: Coordinate,options: AMap[object]) -> None:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        functionArguments.add(position)
        functionArguments.add(options)
        response = self.bridgeFunction.callFunction("setFocusPositionWithOptions",functionArguments,False)

    #
    # Add a new view.
    # @param position The position to focus on (either 2D or 3D).
    # @param renderMode The render mode of the view to open. Have a look at the various `RENDER_MODE_ERROR_` values in the `Constants` class.
    # @param closeExistingViews Should all existing views be closed?
    # @return an error message, see the various `ADD_VIEW_ERROR_` values in the `Constants` class.
    #
    def addView(self,position: Coordinate,renderMode: str,closeExistingViews: bool) -> Promise[str]:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        functionArguments.add(position)
        functionArguments.add(renderMode)
        functionArguments.add(Boolean.valueOf(closeExistingViews))
        response = self.bridgeFunction.callFunction("addView",functionArguments,True)
        promiseId = response
        promise = Promise[str]()
        promise.setId(promiseId.intValue())
        return promise

    #
    # Add a new view.
    # @param position The position to focus on (either 2D or 3D).
    # @param renderMode The render mode of the view to open. Have a look at the various `RENDER_MODE_ERROR_` values in the `Constants` class.
    # @param closeExistingViews Should all existing views be closed?
    # @return an error message, see the various `ADD_VIEW_ERROR_` values in the `Constants` class.
    #
    def addViewWithOptions(self,position: Coordinate,renderMode: str,closeExistingViews: bool,options: AMap[object]) -> Promise[str]:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        functionArguments.add(position)
        functionArguments.add(renderMode)
        functionArguments.add(Boolean.valueOf(closeExistingViews))
        functionArguments.add(options)
        response = self.bridgeFunction.callFunction("addViewWithOptions",functionArguments,True)
        promiseId = response
        promise = Promise[str]()
        promise.setId(promiseId.intValue())
        return promise

    #
    # Communicate a position that was picked by the user on the host reference map to the viewer.
    # Calling this function only has effect when it is called after the `onPickPositionRequested` signal has been received.
    # @param position The user-chosen position on the host reference map.
    #
    def setPickPosition(self,position: Coordinate) -> None:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        functionArguments.add(position)
        response = self.bridgeFunction.callFunction("setPickPosition",functionArguments,False)

    #
    # Adds a list of objects as overlays on the viewer.
    # A resource for holding these map objects will be added to the workspace of the viewer if it does not exist yet.
    # @param list a list of objects to add.
    #
    def addOverlayObjects(self,list: AList[MapObject]) -> None:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        functionArguments.add(list)
        response = self.bridgeFunction.callFunction("addOverlayObjects",functionArguments,False)

    #
    # Clear the list of overlay objects from the viewer.
    #
    def clearOverlayObjects(self) -> None:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        response = self.bridgeFunction.callFunction("clearOverlayObjects",functionArguments,False)

    #
    # Transform a coordinate to a different CRS.
    # @param coordinate The coordinate to transform.
    # @param targetCRS The target EPSG code.
    # @return Coordinate The coordinate expressed in the new CRS.
    #
    def transformCoordinate(self,coordinate: Coordinate,targetCRS: str) -> Coordinate:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        functionArguments.add(coordinate)
        functionArguments.add(targetCRS)
        response = self.bridgeFunction.callFunction("transformCoordinate",functionArguments,True)
        return response

    #
    # Get a property.
    # Note: This function is only for internal SDK use, application builders do not have to call it.
    # @param propertyName The name of the property.
    # @return Returns the property value.
    #
    def getProperty(self,propertyName: str) -> object:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        functionArguments.add(propertyName)
        response = self.bridgeFunction.callFunction("getProperty",functionArguments,True)
        return response

    #
    # Set a property.
    # Note: This function is only for internal SDK use, application builders do not have to call it.
    # @param propertyName The name of the property.
    # @param propertyValue The value of the property.
    #
    def setProperty(self,propertyName: str,propertyValue: object) -> None:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        functionArguments.add(propertyName)
        functionArguments.add(propertyValue)
        response = self.bridgeFunction.callFunction("setProperty",functionArguments,False)

    #
    # Get the version of the javascript SDK.
    # @return the version.
    #
    def getSDKVersion(self) -> str:
        return self.getProperty("JS_SDK_VERSION")

    #
    # Get the date of the javascript SDK.
    # @return the date.
    #
    def getSDKDate(self) -> str:
        return self.getProperty("JS_SDK_DATE")

    #
    # Perform an action.
    # Note: This function is only for internal SDK use, application builders do not have to call it.
    # @param action The action to perform.
    # @param argumentList The arguments.
    # @return Returns the result.
    #
    def doAction(self,action: str,argumentList: AList[object]) -> object:
        functionArguments = AList[object](Collections.TYPE_OBJECT)
        functionArguments.add(action)
        functionArguments.add(argumentList)
        response = self.bridgeFunction.callFunction("doAction",functionArguments,True)
        return response

    #
    # Dispatch an incoming signal value.
    # @param signalName the name of the signal.
    # @param serializedValue the serialized values of the signal.
    #
    def dispatchSignal(self,signalName: str,serializedValue: str) -> None:
        # deserialize the value
        value = None
        if ((serializedValue!=None) and (Strings.getLength(serializedValue)>0)): value = SerializeUtil.deserialize(serializedValue)
        # dispatch
        if (Strings.equals(signalName,"onAppStateChanged")): self.onAppStateChanged.dispatchObject(value)
        if (Strings.equals(signalName,"onFootprintsChanged")): self.onFootprintsChanged.dispatchObject(value)
        if (Strings.equals(signalName,"onFieldOfViewsChanged")): self.onFieldOfViewsChanged.dispatchObject(value)
        if (Strings.equals(signalName,"onGlobalsChanged")): self.onGlobalsChanged.dispatchObject(value)
        if (Strings.equals(signalName,"onMeasureStateChanged")): self.onMeasureStateChanged.dispatchObject(value)
        if (Strings.equals(signalName,"onZoomToExtent")): self.onZoomToExtent.dispatchObject(value)
        if (Strings.equals(signalName,"onFocusPositionChanged")): self.onFocusPositionChanged.dispatchObject(value)
        if (Strings.equals(signalName,"onPickPositionRequested")): self.onPickPositionRequested.dispatchObject(value)
        if (Strings.equals(signalName,"onPickPositionCancelled")): self.onPickPositionCancelled.dispatchObject(value)
        if (Strings.equals(signalName,"onOverlayObjectChanged")): self.onOverlayObjectChanged.dispatchObject(value)
        if (Strings.equals(signalName,"onOverlayObjectDeleted")): self.onOverlayObjectDeleted.dispatchObject(value)
        if (Strings.equals(signalName,"onOverlayObjectCreated")): self.onOverlayObjectCreated.dispatchObject(value)
