# NOTE: this class has been translated from Java.
# Do not modify this file, changes will be overwritten.

#package orbitgt.mapping3d.sdk.viewer;

# haxe remove begin
from .....com.orbitgis.lib.lang.StringBuilder import StringBuilder
# haxe remove end
from .....com.orbitgis.tool.Strings import Strings
from .....orbitgt.mapping3d.sdk.viewer.AList import AList
from .....orbitgt.mapping3d.sdk.viewer.Bounds import Bounds
from .....orbitgt.mapping3d.sdk.viewer.Coordinate import Coordinate
from .....orbitgt.mapping3d.sdk.viewer.CoordinatePart import CoordinatePart

#
# Class CoordinatePath defines a spatial path as a list of parts, so it can be a multi-polyline or a multi-polygon.
#
# Donut shapes can be created by adding multiple parts to a single path.
# The winding rule for filling is 'nonzero', see https://en.wikipedia.org/wiki/Nonzero-rule
# So take care of the (counter)clockwise direction of each part.
#

class CoordinatePath:
    # The type of objects in this class */
    TYPE: str = "CP"

    # The EPSG CRS code of the coordinates in this path, like "4326" for example */
    #public String crs
    # The list of parts */
    #public AList[CoordinatePart] parts

    #
    # Create a new path. Be sure to set the `crs` property when adding parts.
    #
    def __init__(self):
        self.crs = None
        self.parts = AList[CoordinatePart](CoordinatePart.TYPE)

    #
    # Get the bounds.
    # @return returns a Bounds instance, never null.
    #
    def getBounds(self) -> Bounds:
        bounds = Bounds(self.crs)
        for i in range(0,self.parts.size()):
            part = self.parts.get(i)
            for j in range(0,part.points.size()):
                bounds.addCoordinate(part.points.get(j))
        return bounds

    #
    # Convert the object to a string.
    # @return the string form of the object.
    #
    def toString(self) -> str:
        return "[CoordinatePath:crs="+self.crs+",partCount="+Strings.toStringInt(self.parts.size())+"]"

    #
    # Convert to a polyline WKT string.
    # @return a WKT string.
    #
    def asPolylineWKT(self) -> str:
        if (self.parts.size()==1):
            wkt = StringBuilder()
            wkt.append("LineString ")
            is3D = self.parts.get(0).points.get(0).is3D
            if (is3D): wkt.append("Z ")
            wkt.append("(")
            part = self.parts.get(0)
            for i in range(0,part.points.size()):
                if (i>0): wkt.append(",")
                point = part.points.get(i)
                if (is3D): wkt.append(""+Strings.toStringDouble(point.x)+" "+Strings.toStringDouble(point.y)+" "+Strings.toStringDouble(point.z))
                else: wkt.append(""+Strings.toStringDouble(point.x)+" "+Strings.toStringDouble(point.y))
            wkt.append(")")
            return wkt.toString()
        if (self.parts.size()>1):
            wkt = StringBuilder()
            wkt.append("MultiLineString ")
            is3D = self.parts.get(0).points.get(0).is3D
            if (is3D): wkt.append("Z ")
            wkt.append("(")
            for p in range(0,self.parts.size()):
                if (p>0): wkt.append(",")
                wkt.append("(")
                part = self.parts.get(p)
                for i in range(0,part.points.size()):
                    if (i>0): wkt.append(",")
                    point = part.points.get(i)
                    if (is3D): wkt.append(""+Strings.toStringDouble(point.x)+" "+Strings.toStringDouble(point.y)+" "+Strings.toStringDouble(point.z))
                    else: wkt.append(""+Strings.toStringDouble(point.x)+" "+Strings.toStringDouble(point.y))
                wkt.append(")")
            wkt.append(")")
            return wkt.toString()
        return "MultiLineString Empty"

    #
    # Convert to a polygon WKT string.
    # @return a WKT string.
    #
    def asPolygonWKT(self) -> str:
        if (self.parts.size()>0):
            wkt = StringBuilder()
            wkt.append("Polygon ")
            is3D = self.parts.get(0).points.get(0).is3D
            if (is3D): wkt.append("Z ")
            wkt.append("(")
            for p in range(0,self.parts.size()):
                if (p>0): wkt.append(",")
                wkt.append("(")
                part = self.parts.get(p)
                for i in range(0,part.points.size()):
                    if (i>0): wkt.append(",")
                    point = part.points.get(i)
                    if (is3D): wkt.append(""+Strings.toStringDouble(point.x)+" "+Strings.toStringDouble(point.y)+" "+Strings.toStringDouble(point.z))
                    else: wkt.append(""+Strings.toStringDouble(point.x)+" "+Strings.toStringDouble(point.y))
                wkt.append(")")
            wkt.append(")")
            return wkt.toString()
        return "MultiPolygon Empty"
