import time
from typing import Callable,Dict,Generic,List,TypeVar

T = TypeVar('T')

class Promise(Generic[T]):
    _INSTANCES: Dict[int,object] = {}
    _VALUES: Dict[int,object] = {}
    _FAIL_VALUES: Dict[int,object] = {}
    
    def popInstance(id: int) -> object:
        if (id in Promise._INSTANCES): return Promise._INSTANCES.pop(id)
        return None

    def storeResult(id: int,value: object) -> None:
        Promise._VALUES[id] = value

    def storeFailResult(id: int,value: object) -> None:
        Promise._FAIL_VALUES[id] = value
        
    def __init__(self) -> None:
        self.id = -1 # int
        self.state = 0 # int
        self.value = None # T
        self.failValue = None # object
        self.resultListeners = []
        self.failListeners = []

    def getId(self) -> int:
        return self.id

    def setId(self,id: int) -> None:
        self.id = id
        if (id in Promise._VALUES):
            self.setResolved(Promise._VALUES.pop(id))
            return
        if (id in Promise._FAIL_VALUES):
            self.setFailed(Promise._FAIL_VALUES.pop(id))
            return
        Promise._INSTANCES[id] = self

    def isPending(self) -> bool:
        return (self.state==0)

    def isResolved(self) -> bool:
        return (self.state==1)

    def setResolved(self,value: T) -> None:
        self.value = value
        self.state = 1
        for listener in self.resultListeners: listener(value)

    def isFailed(self) -> bool:
        return (self.state==2)

    def setFailed(self,value: object) -> None:
        self.failValue = value
        self.state = 2
        for listener in self.failListeners: listener(value)

    def then(self,resultListener: Callable[[T],None],failListener:  Callable[[object],None]) -> None:
        if self.isResolved():
            if (resultListener!=None): resultListener(self.value)
        elif self.isFailed():
            if (failListener!=None): failListener(self.failValue)
        else:
            if (resultListener!=None): self.resultListeners.append(resultListener)
            if (failListener!=None): self.failListeners.append(failListener)

    def catchError(self,failListener: Callable[[object],None]) -> None:
        self.then(None,failListener)

    def waitFor(self) -> T:
        iterations = 0
        while True:
            if (self.state==1): return self.value
            if (self.state==2): raise Exception("Promise ["+str(self.id)+"] waitFor failed: "+self.failValue)
            time.sleep(0.1)
            iterations += 1
            if (iterations==300): raise Exception("Promise ["+str(self.id)+"] waitFor timeout expired")
